//------------------------------------------------------------------------------------------------------
//
//  ToothPIC.c - ToothPIC settings and link key map 
//
//  You will need a copy of this for each application you develop,
//  so copy it from the Development Kit directory to your application
// development directory and modify it there
//
//  (c) FlexiPanel Ltd
//
//-------------------------------------------------------------------------------------------------------
 
#define __ToothPIC_c__ 
#include "..\ToothPIC.h"
#include <p18f6720.h>

// If programming a FlexiPanel UI from FlexiPanel Designer using data files, include the header
// file Designer creates, (comment out otherwise)
#include "ToothPICTestRes.h" 

//-----------------------------------------------------------------------------------------

// ToothPIC settings and link key map 
#pragma romdata TP_SLKM=0x00C080

// ToothPIC checks DataValidMarkerBottom value is correct.  If it is not, 'Safe' values will overwritten as follows:
// pLocalName -> "ToothPIC 3.0"
// pszPIN -> "0000";
// LinkKeyMax -> 0
rom unsigned short DataValidMarkerBottom = USHORT_DVMB;

//-------------------------sec-min-hour-date-dow-month-year
rom DateTimeU RESETTIME = {	00, 00, 00,  01,  6,   01, 2000 };
rom unsigned char DSTDEFAULT = DST_NONE;

/* In versions 3.0.00001 - 3.0.00004, DeviceClass was incorrectly 2 bytes long */
/* To maintain backward compatability, the two bytes where the value was stored will be maintained as a valid device class value */
rom unsigned short OldDeviceClass =   Svc_Phone | Svc_Object | Svc_Network | Svc_LtdDisc | Dev_Ph_Cell; 

// ToothPICSettings flags - OR together TPS_values 
// TPS_RealTimeClock	- implement real time clock
// TPS_ClientTick	  	- send tick event messages to LowInterrupt
// TPS_RegisterHeadsetServices - be visible to all mobile phones, even those that do not support serial port
// TPS_AccessKeyOnHardProgram - require the access key for wireless programming initiated by button press at power up - CAUTION, may render the module non-upgradeable by wireless
// TPS_PermitSoftProgram - allows wireless field programmer to initiate WFP by connecting to FlexiPanel server 
rom unsigned char ToothPICSettings = TPS_RealTimeClock | TPS_ClientTick | TPS_PermitSoftProgram ;

/* You can change the name (up to 31 characters) but do NOT change the maximum length LOCALNAMELEN*/
/* The name must be ASCII.   You can also change the name at runtime by writing */
/* new name here (remember the zero terminator). Change takes effect after reset*/
/* The DESIGNERCHOOSESNAME overrides the name during development (with Data */
/* file target) when the Force layout flag is set in Designer */
#ifndef DESIGNERCHOOSESNAME
rom unsigned char pLocalName[LOCALNAMELEN] = "ToothPIC 3.0";    
#else
rom unsigned char pLocalName[LOCALNAMELEN] = DESIGNERCHOSENNAME;    
#endif

/* ToothPIC will set this value to the attached BluePort's Bluteooth address */
rom unsigned char pLocalBTAddr[BTADDRLEN]; 

/* ToothPIC will set this to a zero terminated ASCII software version numner*/
rom unsigned char pVersionStr[VERSTRLEN]; 

/* You can set the authentication PIN to what you want. You can also change the PIN at */
/* runtime by writing a new zero terminated ASCII PIN here (max 16 ASCII characters plus */
/* zero terminator).  Bear in mind some devices (phones) can only enter PIN codes with */
/* digits 0-9.  For no authentication, use the command BMTC_Security rather than */
/* setting the PIN to zero length.  The default PIN code is the Bluetooth de facto default 0000 */
rom char pszPIN[SZPINMAX] = "0000";

/* You can set the wireless field programmer access key to what you want. You can also change the */
/* code at runtime by writing a new zero terminated ASCII string here (max 7 ASCII characters plus */
/* zero terminator. For no access key, specify the null string "" */
rom char pszWFPAccessKey[SZACCESSMAX] = "Access";

/* You can set DeviceClass to any class of device you wish so long as it confirms to the specification */
/* document "Assigned Numbers - Bluetooth Baseband" available from blurtooth.org */
/* Devices classes defined when this comment was written are specified in ToothPIC.h */
/* in the section Bluetooth Device Class constants - combine as follows:			*/
/*   - as many Svc flags as appropriate 											*/
/*   - exactly one Dev_ value 														*/
/*   - if the Dev_ value was a Dev_Pr_ value, one PrDev_ value also					*/
/*   - if the Dev_ value was Dev_Imaging value, one or more ImDev_ value also		*/
rom unsigned short long DeviceClass =   Svc_Phone | Svc_Object | Svc_Network | Svc_LtdDisc | Dev_Ph_Cell; 

/* Set LINKKEYMAX to the number of paired devices you can store 1-255 */
/* Change the #define only - if LinkKeyMax doesn't match number of LinkKeys */
/* you will overwrite your own code */
// BlueProxy - set to zeroto disable link key storage.  PIN key must
// always be specified
#define LINKKEYMAX 24
rom unsigned char LinkKeyMax = LINKKEYMAX;
#if  LINKKEYMAX > 0
rom LinkRecord pLinkKey[LINKKEYMAX];
#else
rom LinkRecord *pLinkKey = 0;
#endif

// ToothPIC checks DataValidMarkerTop value is correct.  If it is not, 'Safe' values will overwritten as follows:
// pLocalName -> "ToothPIC 3.0"
// pszPIN -> "0000";
// LinkKeyMax -> 0
rom unsigned short DataValidMarkerTop = USHORT_DVMT;

#pragma romdata 

//-----------------------------------------------------------------------------------------
// BlueMatik buffer

#define BMTRXBUFFLEN 256		// do NOT change - circular buffer assumes length of 256
#define BMTTXBUFFLEN 256		// do NOT change - circular buffer assumes length of 256

#pragma udata BMTRXBUFF
volatile unsigned char BMTRxBuff[BMTRXBUFFLEN];
#pragma udata BMTTXBUFF
volatile unsigned char BMTTxBuff[BMTTXBUFFLEN];
#pragma udata 

//-----------------------------------------------------------------------------------------

// public RAM data definitions; data larger than char are double word aligned
#pragma udata TOOTHPICD
volatile DateTimeU 		RealTimeClock;				// Real Time Clock, seconds to years
volatile unsigned char 	DSTEvent;			  		// Daylight Savings Time rule in effect
volatile unsigned char 	ToothPICSemaphores;  		// OS status semaphores
volatile unsigned char 	ToothPICSemaphores2;  		// OS status semaphores
volatile unsigned char 	SWIflags;  					// event(s) which triggered software interrupt
volatile unsigned char 	BMTRxStart;					// Receive buffer startpoint
volatile unsigned char 	BMTRxEnd;					// Receive buffer endpoint
volatile unsigned char 	BMTTxStart;					// Transmit buffer startpoint
volatile unsigned char 	BMTTxEnd;					// Transmit buffer endpoint
volatile unsigned char 	pRemoteBTAddr[BTADDRLEN]; 	// bluetooth address of remote device
unsigned char usDelayCounter; 	

