//------------------------------------------------------------------------------------------------------
//
//  ToothPICTest.c
//
//  ToothPIC test higher-level functions service
//
//  (c) FlexiPanel Ltd
//
//-------------------------------------------------------------------------------------------------------

#include "..\ToothPIC.h"
#include <p18f6720.h>
#include "ToothPICTestRes.h"
#include "..\hopcodes.h"

//#define RELEASE_VERSION 1  /* Uncomment for release versions - greater robustness, less error reporting */

// Semaphores
unsigned char MySemaphores = 0x00;
#define MSF_RunTestNow		0x01		// set once per second if a test is running
#define MSF_TestsRunning	0x02  	// set if a test is running 

// Static Data
rom unsigned char TrueVal = 0xFF;
rom unsigned char FalseVal = 0x00;
rom unsigned long ColRed = 0x000000FF;
rom unsigned long ColGray = 0x00C0C0C0;

// Prototypes for functions in this source file
void SetDigitalOutputs( void );
void SetParallelOutputs( void );
void SetLEDs( void );
void GetMemVal( unsigned char Storage, unsigned char * pLocText, unsigned char * pValText, unsigned short CtlID );
void SetMemVal( unsigned char Storage, unsigned char * pLocText, unsigned char * pValText, unsigned short CtlID );

// main program loop
void main( void )
{
	overlay long lVal;
	overlay short Row, Col, sVal;
	overlay char cVal;

	// if no BlueMatik, flash red led rapidly
	while ((ToothPICSemaphores&TPSF_BMTEXISTS)==0)
	{
		LedRed = ~LedRed;
		msDelay(50);
	}

	// initialize external memory I/O; this is 'safe' and will not crash if there is not external memory
	I2CMemInit400kHz;

	FxPCommand( FxPC_Start, 0, 0 );		// Start FlexiPanel service
	AwaitBMTOK; 						// Service operating, no client

	// put data in list matrix; each cell is 1 byte
  	for (Row = 0; Row < MaxRow_List_Matrix_EA; Row++)  
	{
		for (Col = 0; Col < NumCol_List_Matrix_EA; Col++)  
		{
			cVal = (Row+1) * (Col+1);
   			*pCell_List_Matrix_EA( Row, Col ) = cVal;
		}
	}
	*pRowCounter_List_Matrix_EA = MaxRow_List_Matrix_EA;

	// put 5 rows of data in XY matrix; each cell is 2 bytes; row value is 4 bytes
  	for (Row = 0; Row < 5; Row++)  
	{
		for (Col = 0; Col < NumCol_XY_Matrix_Section_EB; Col++)  
		{
			sVal = (Row+1)  + Col*2;
   			*pCell_XY_Matrix_Section_EB( Row, Col ) = sVal;
		}
		lVal = Row;
		*pRow_XY_Matrix_Section_EB( Row ) = lVal;
	}
	*pRowCounter_XY_Matrix_Section_EB = 5;

	// put data in labels matrix; each cell is 4 bytes; only one column
  	for (Row = 0; Row < MaxRow_Labels_Matrix_EC; Row++)  
	{
		lVal = Row * Row + 50;
   		*pCell_Labels_Matrix_EC( Row, 0 ) = lVal;
	}
	*pRowCounter_Labels_Matrix_EC = MaxRow_Labels_Matrix_EC;

	// main loop
	while ( 1 ) 
	{
		// Run tests and flash LED every 250ms
		msDelay( 250 );

		// Flash alive if not doing LED test
		if ( !( (*psTest_2 + 1 == DlgID_PB_LED_test_A5) && ( MySemaphores & MSF_TestsRunning ) ) )
		{
			LedGreen = ~LedGreen;
		}

		// run tests intermittently
		if ( IsFxPInitialized && ( MySemaphores & MSF_RunTestNow ) )
		{
			MySemaphores &= ~MSF_RunTestNow;
			switch (*psTest_2 + 1)  // switch on currently active dialog
			{
			default:
				// do nothing
				break;

			case DlgID_AD_Test_C:
				{
					// analog to digital measurment
					unsigned long ADResult;
					SetADChan( *pcADselect_D );  			// select a to d channel
					CyclesDelay3p2plus3p2times( 4 ); 		// 16us delay to charge S & H cap (refer to PIC18F6720 manual)
					StartAtoD;								// start a to d conversion
					AwaitAtoDComplete;						// await end of conversion
					GetADResult10bit( ADResult );			// get result
					ADResult = 	(ADResult * 5000) / 1024; 	// convert to millivolts				
					SetUp_Number24_E( 0, &ADResult );
					break;
				}

			case DlgID_Digital_Inputs_2A:
				{
					unsigned long lVal[2]; 
					lVal[0] = 0x00000040; 
					lVal[1] = AN0Pin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_AN0_62, lVal );
					lVal[1] = AN1Pin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_AN1_64, lVal );
					lVal[1] = AN2Pin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_AN2_65, lVal );
					lVal[1] = AN3Pin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_AN3_67, lVal );
					lVal[1] = AN4Pin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_AN4_66, lVal );
					lVal[1] = AN5Pin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_AN5_68, lVal );
					lVal[1] = AN6Pin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_AN6_69, lVal );
					lVal[1] = AN7Pin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_AN7_6A, lVal );
					lVal[1] = AN8Pin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_AN8_6B, lVal );
					lVal[1] = AN9Pin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_AN9_6C, lVal );
					lVal[1] = AN10Pin ? ColRed : ColGray;		FxPCommand( FxPC_SetProps, ID_AN10_6D, lVal );
					lVal[1] = AN11Pin ? ColRed : ColGray;		FxPCommand( FxPC_SetProps, ID_AN11_6E, lVal );
					lVal[1] = CCP1Pin ? ColRed : ColGray;		FxPCommand( FxPC_SetProps, ID_CCP1_70, lVal );
					lVal[1] = CCP2Pin ? ColRed : ColGray;		FxPCommand( FxPC_SetProps, ID_CCP2_71, lVal );
					lVal[1] = CCP3Pin ? ColRed : ColGray;		FxPCommand( FxPC_SetProps, ID_CCP3_73, lVal );
					lVal[1] = CCP4Pin ? ColRed : ColGray;		FxPCommand( FxPC_SetProps, ID_CCP4_72, lVal );
					lVal[1] = CCP5Pin ? ColRed : ColGray;		FxPCommand( FxPC_SetProps, ID_CCP5_74, lVal );
					lVal[1] = SDAPin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_SDA_77, lVal );
					lVal[1] = SCLPin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_SCL_78, lVal );
					lVal[1] = SDOPin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_SDO_79, lVal );
					lVal[1] = TxDPin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_TxD_7A, lVal );
					lVal[1] = RxDPin ? ColRed : ColGray; 		FxPCommand( FxPC_SetProps, ID_RxD_7B, lVal );
					break;
				}
			case DlgID_Parallel_Inputs_7E:
				{
				long lVal;

				// Parallel input A
				switch (*pcPAN_86)
				{
					case 0:  // PA7
						DirPA7( DirInput );
						lVal = GetPA7;
						SetUp_PAVal_87( 0, &lVal );
						break;
					case 1:  // PA6
						DirPA6( DirInput );
						lVal = GetPA6;
						SetUp_PAVal_87( 0, &lVal );
						break;
					case 2:  // PA5
						DirPA5( DirInput );
						lVal = GetPA5;
						SetUp_PAVal_87( 0, &lVal );
						break;
					case 3:  // PA4
						DirPA4( DirInput );
						lVal = GetPA4;
						SetUp_PAVal_87( 0, &lVal );
						break;
					case 4:  // PA3
						DirPA3( DirInput );
						lVal = GetPA3;
						SetUp_PAVal_87( 0, &lVal );
						break;
					case 5:  // PA2
						DirPA2( DirInput );
						lVal = GetPA2;
						SetUp_PAVal_87( 0, &lVal );
						break;
				}
			
				// Parallel input B
				switch (*pcPBN_88)
				{
					case 0:  // PB5
						DirPB5( DirInput );
						lVal = GetPB5;
						SetUp_PBVal_89( 0, &lVal );
						break;
					case 1:  // PB4
						DirPB4( DirInput );
						lVal = GetPB4;
						SetUp_PBVal_89( 0, &lVal );
						break;
					case 2:  // PB3
						DirPB3( DirInput );
						lVal = GetPB3;
						SetUp_PBVal_89( 0, &lVal );
						break;
					case 3:  // PB2
						DirPB2( DirInput );
						lVal = GetPB2;
						SetUp_PBVal_89( 0, &lVal );
						break;
				}
			
			
				// Parallel input C
				switch (*pcPCN_8A)
				{
					case 0:  // PC5
						DirPC5( DirInput );
						lVal = GetPC5;
						SetUp_PCVal_8B( 0, &lVal );
						break;
					case 1:  // PC4
						DirPC4( DirInput );
						lVal = GetPC4;
						SetUp_PCVal_8B( 0, &lVal );
						break;
					case 2:  // PC3
						DirPC3( DirInput );
						lVal = GetPC3;
						SetUp_PCVal_8B( 0, &lVal );
						break;
					case 3:  // PC2
						DirPC2( DirInput );
						lVal = GetPC2;
						SetUp_PCVal_8B( 0, &lVal );
						break;				
				}
				}

			}  // end of switch (*psTest_2 + 1)

		}  // end of if ( IsFxPInitialized && ( MySemaphores & MSF_RunTestNow ) )

	}  // end of while ( 1 ) 

}

// HighInterrupt handler
void HighInterrupt (void)
{
}

// LowInterrupt handler
void LowInterrupt (void)
{
	if (IsSWI( SWI_Tick ) )
	{
		// must take care not to process ticks before FlexiPanel service has started
	    if ( IsFxPInitialized )
	    {
			// update RTC fixed dialog 
			rom char * pNewText;
			SetUp_DT_Fixed_7( 0, (unsigned char *) &RealTimeClock );

			// update day of week dialog
			switch (RealTimeClock.dow)
			{
				case 0: pNewText = "Sun";  break;
				case 1: pNewText = "Mon";  break;
				case 2: pNewText = "Tue";  break;
				case 3: pNewText = "Wed";  break;
				case 4: pNewText = "Thu";  break;
				case 5: pNewText = "Fri";  break;
				case 6: pNewText = "Sat";  break;
			}
			// update control if changed
			if (pNewText[0] != pDayOfWeek_C4[0] || pNewText[1] != pDayOfWeek_C4[1])  SetUp_DayOfWeek_C4( pNewText, 0 );

			// update summertime dialog
			if (*pSummertime_C3 == 0)  // winter
			{
				if (DSTEvent & DST_SUMMER_FLAG)  SetUp_Summertime_C3( &TrueVal, 0 );
			}
			else
			{
				if ( (DSTEvent & DST_SUMMER_FLAG)==0 )  SetUp_Summertime_C3( &FalseVal, 0 );
			}
					
			if ( RealTimeClock.sec % 5 == 0 )
			{
			  	// log point in date time matrix every 5 secs 
				unsigned short RowToUpdate = AppendRow_Time_Matrix_ED;
		      	char NewVal = (RealTimeClock.sec % 4) + RealTimeClock.sec / 4;
		      	Set_Time_Matrix_ED_Cell( 0, &NewVal, RowToUpdate, 0 );
		      	Set_Time_Matrix_ED_Row( 0, (unsigned char *)&RealTimeClock, RowToUpdate );
		      	NextAppendRow_Time_Matrix_ED;
		      	FxPCommand( FxPC_PartUpdate, ID_Time_Matrix_ED, (void *) RowToUpdate );
		    }
	
			// raise semaphore to run intermittent test - preferable to running test in callback
			if (MySemaphores & MSF_TestsRunning) MySemaphores |= MSF_RunTestNow;
		}

		ClearSWI( SWI_Tick );		// Clear interrupt flag

		return;
	}

	// pushbutton interrupt - show message
	if (PushbuttonIntFlag)
	{
		unsigned long lVal;

		// set message text
		SetUp_Message_CA( 0, pMessageText_AD );

		// set message style
		// CTL_MSG_ICON_NONE etc are defined in hopcodes.h 
		switch (*pcIcon_A8)
		{
			case 0: // none
				lVal = CTL_MSG_ICON_NONE;   break;
			case 1: // stop
				lVal = CTL_MSG_ICON_STOP;   break;
			case 2: // exclamation
				lVal = CTL_MSG_ICON_EXCLAMATION;   break;
			case 3: // question
				lVal = CTL_MSG_ICON_QUESTION;   break;
			case 4: // information
				lVal = CTL_MSG_ICON_INFORMATION;   break;
		}

		// set message style
		switch (*pcResponse_type_A9)
		{
			case 0: // none
				lVal |= CTL_MSG_RESP_NONE;   break;
			case 1: // OK
				lVal |= CTL_MSG_RESP_OK;   break;
			case 2: // retry cancel
				lVal |= CTL_MSG_RESP_OKCANCEL;   break;
			case 3: // yes no
				lVal |= CTL_MSG_RESP_RETRYCANCEL;   break;
			case 4: // yes no cancel
				lVal |= CTL_MSG_RESP_YESNO;   break;
			case 5: // yes no cancel
				lVal |= CTL_MSG_RESP_YESNOCANCEL;   break;
			case 6: // yes no cancel
				lVal |= CTL_MSG_RESP_ABORTRETRYIGNORE;   break;
		}

		// show message - the lack of a CTL_INVISIBLE flag implies that the message should be shown
		FxPCommand( FxPC_SetProps, ID_Message_CA, &lVal );

		// clear interrtupt flag
		PushbuttonIntFlag = 0;
	}
}


// Error event handler 
void ErrorStatus( unsigned char ErrNum )
{
#ifndef RELEASE_VERSION
	// During development - Flash error number for diagnositic purposes
	Breakpoint( ErrNum );
#else
	// Product release - best policy for unanticipated errors is to reset
	Reset();
#endif
}


// BlueMatik Event handler 
void BMTEvent( unsigned char EventID, void *pData1, void *pData2 )
{
	if ( EventID==BMTE_Error || EventID==BMTE_Warning )
	{
#ifndef RELEASE_VERSION
	// During development - Flash error number for diagnositic purposes
	Breakpoint( ((unsigned char*)pData1)[0] );
#else
	// Product release - best policy for unanticipated errors is to reset
	Reset();
#endif
	}

}

// FxPEvent handler 
void FxPEvent( unsigned char EventID, void *pData )
{
	if ( EventID==FxPE_Connect ) 
	{
		// turn off red led during connection
		LedRed = LedRedOn;
	}

	else if ( EventID==FxPE_Disco ) 
	{
		// turn off red led after disconnection & stop running tests
		LedRed = LedRedOff;
		MySemaphores &= ~MSF_TestsRunning;
	}

	else if ( EventID==FxPE_ClntUpdate ) 
	{
		// main dialog

		if (*((unsigned short*) pData) == ID_Close_5 )
		{
			// close button presed - disconnect
			FxPCommand( FxPC_Disco, 0, 0 );	
			// turn off red led after disconnection & stop running tests
			LedRed = LedRedOff;
			MySemaphores &= ~MSF_TestsRunning;
		}

		else if (*((unsigned short*) pData) == ID_Run_test_3)
		{
			unsigned char NewDialog = (*psTest_2 )+1;

			// switch to appropriate test dialog
			FxPCommand( FxPC_SetDialog, NewDialog, 0 );	

			// if analog to digital test, set I/O to analog inputs
			if ( NewDialog == DlgID_AD_Test_C )
			{
				SetAnalogAD0toAD11;
				ADConverterOn10bit;
				VRefNegIsVss;
				VRefPosIsVdd;
				MySemaphores |= MSF_TestsRunning;
			}

			// if digital input test or parallel input test, set I/O to digital inputs
			if ( NewDialog == DlgID_Digital_Inputs_2A || NewDialog == DlgID_Parallel_Inputs_7E )
			{
				SetAnalogNone;
				DirAN0 = DirInput;
				DirAN1 = DirInput;
				DirAN2 = DirInput;
				DirAN3 = DirInput;
				DirAN4 = DirInput;
				DirAN5 = DirInput;
				DirAN6 = DirInput;
				DirAN7 = DirInput;
				DirAN8 = DirInput;
				DirAN9 = DirInput;
				DirAN10 = DirInput;
				DirAN11 = DirInput;
				DirCCP1 = DirInput;
				DirCCP2 = DirInput;
				DirCCP3 = DirInput;
				DirCCP4 = DirInput;
				DirCCP5 = DirInput;
				DirINT0 = DirInput;
				DirINT1 = DirInput;
				DirSCL = DirInput;
				DirSDA = DirInput;
				DirSDO = DirInput;
				DirTxD = DirInput;
				DirRxD = DirInput;
				MySemaphores |= MSF_TestsRunning;
			}

			// if digital output test or parallel output test, set I/O to digital outputs and set output values
			if ( NewDialog == DlgID_Digital_Outputs_10 || NewDialog == DlgID_Parallel_Outputs_7C )
			{
				SetAnalogNone;
				DirAN0 = DirOutput;
				DirAN1 = DirOutput;
				DirAN2 = DirOutput;
				DirAN3 = DirOutput;
				DirAN4 = DirOutput;
				DirAN5 = DirOutput;
				DirAN6 = DirOutput;
				DirAN7 = DirOutput;
				DirAN8 = DirOutput;
				DirAN9 = DirOutput;
				DirAN10 = DirOutput;
				DirAN11 = DirOutput;
				DirCCP1 = DirOutput;
				DirCCP2 = DirOutput;
				DirCCP3 = DirOutput;
				DirCCP4 = DirOutput;
				DirCCP5 = DirOutput;
				DirINT0 = DirOutput;
				DirINT1 = DirOutput;
				DirSCL = DirOutput;
				DirSDA = DirOutput;
				DirSDO = DirOutput;
				DirTxD = DirOutput;
				DirRxD = DirOutput;
				if (NewDialog == DlgID_Digital_Outputs_10)
					SetDigitalOutputs();
				else
					SetParallelOutputs();
			}

			// if LED and Pushbutton, initialize LEDs and interrupt
			if ( NewDialog == DlgID_PB_LED_test_A5 )
			{
				MySemaphores |= MSF_TestsRunning;
				SetLEDs( );

				EnablePBIntLowDown;				
			}

		}

		// end test button pressed - return to initial state
		else if (*((unsigned short*) pData) == ID_End_Test_B ||
				*((unsigned short*) pData) == ID_End_Test_F ||
				*((unsigned short*) pData) == ID_End_Test_29 ||
				*((unsigned short*) pData) == ID_End_Test_2B ||
				*((unsigned short*) pData) == ID_End_Test_7D ||
				*((unsigned short*) pData) == ID_End_Test_7F ||
				*((unsigned short*) pData) == ID_End_Test_93 ||
				*((unsigned short*) pData) == ID_End_Test_9A ||
				*((unsigned short*) pData) == ID_End_Test_A1 ||
				*((unsigned short*) pData) == ID_End_Test_A4 ||
				*((unsigned short*) pData) == ID_End_Test_A6 ||
				*((unsigned short*) pData) == ID_End_Test_AF ||
				*((unsigned short*) pData) == ID_End_Test_C7 ||
				*((unsigned short*) pData) == ID_End_Test_C8 ||
				*((unsigned short*) pData) == ID_End_Test_FC)
		{
			// close dialog
			FxPCommand( FxPC_SetDialog, 0, 0 );		

			// stop running tests
			MySemaphores &= ~MSF_TestsRunning;

			// LED and PB tests may have set pushbutton interrupts and cleared Red LED - return to normal state
			DisablePBInt;
			LedRed = LedRedOn;
		}

		// rtc test dialog
		else if (*((unsigned short*) pData) == ID_DT_Edit_8 )
		{
			// calculate day of week based on date entered
			CalcDayOfWeek( RealTimeClock.dow, RealTimeClock.year, RealTimeClock.month, RealTimeClock.date );
			// DT Fixed and DayOfWeek get updated every second anyway, so no need to bother here	
		}	

		else if (*((unsigned short*) pData) == ID_DSTStyle_C2 || *((unsigned short*) pData) == ID_Summertime_C3 )
		{
			// update daylight savings time rules
			DSTEvent = *pcDSTStyle_C2;
			if (*pSummertime_C3) DSTEvent |= DST_SUMMER_FLAG;
		}	
		else if (*((unsigned short*) pData) == ID_Refresh_SecFrac_value_A )
		{
			// read sec frac value and place in ID_ScFrc_9
			GetSecFracs( psScFrc_9 );
			FxPCommand( FxPC_CtlUpdate, ID_ScFrc_9, 0 );
		}	
		else if (*((unsigned short*) pData) == ID_ScFrc_9 )
		{
			// set sec frac value to ID_ScFrc_9
			SetSecFracs( psScFrc_9 );
		}	

		// Digital Output dialog
		else if (*((unsigned short*) pData) == ID_AN0_11 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN1_12 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN2_13 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN3_14 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN4_15 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN5_16 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN6_17 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN7_18 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN8_19 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN9_1A ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN10_1B ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN11_1C ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_AN0_11 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_CCP1_1D ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_CCP2_1E ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_CCP3_1F ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_CCP4_20 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_CCP5_21 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_INT0_22 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_INT1_23 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_SCL_26 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_SDA_24 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_SDO_25 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_TxD_27 ) SetDigitalOutputs();
		else if (*((unsigned short*) pData) == ID_RxD_28 ) SetDigitalOutputs();

		// parallel output dialog
		else if (*((unsigned short*) pData) == ID_PAN_84 ) SetParallelOutputs();
		else if (*((unsigned short*) pData) == ID_PAVal_80 ) SetParallelOutputs();
		else if (*((unsigned short*) pData) == ID_PBN_83 ) SetParallelOutputs();
		else if (*((unsigned short*) pData) == ID_PBVal_81 ) SetParallelOutputs();
		else if (*((unsigned short*) pData) == ID_PCN_85 ) SetParallelOutputs();
		else if (*((unsigned short*) pData) == ID_PCVal_82 ) SetParallelOutputs();

		// LED & PB test dialog
		else if (*((unsigned short*) pData) == ID_Green_LED_AB ) SetLEDs();
		else if (*((unsigned short*) pData) == ID_Red_Led_AC ) SetLEDs();

		// memory test dialog; change STR_ROM01 to STR_ROM00 to test ROM locations 0x00000 - 0x00FFFF
		// change STR_EXT2 to STR_EXTx to test Ext memory addresses other than 2
		else if (*((unsigned short*) pData) == ID_Get_Value_CD )  GetMemVal( STR_ROM01, pROMLoc_CC, pROMVal_CE, ID_ROMVal_CE );
		else if (*((unsigned short*) pData) == ID_Set_Value_CF )  SetMemVal( STR_ROM01, pROMLoc_CC, pROMVal_CE, ID_ROMVal_CE );
		else if (*((unsigned short*) pData) == ID_Get_Value_DA )  GetMemVal( STR_RAM, pRAMLoc_DD, pRAMVal_D8, ID_RAMVal_D8 );
		else if (*((unsigned short*) pData) == ID_Set_Value_D0 )  SetMemVal( STR_RAM, pRAMLoc_DD, pRAMVal_D8, ID_RAMVal_D8 );
		else if (*((unsigned short*) pData) == ID_Get_Value_DB )  GetMemVal( STR_EE, pEELoc_DE,  pEEVal_D9, ID_EEVal_D9 );
		else if (*((unsigned short*) pData) == ID_Set_Value_D1 )  SetMemVal( STR_EE, pEELoc_DE,  pEEVal_D9, ID_EEVal_D9 );
		else if (*((unsigned short*) pData) == ID_Get_Value_DC )  GetMemVal( STR_EXT2, pExtLoc_DF, pExtVal_D5, ID_ExtVal_D5 );
		else if (*((unsigned short*) pData) == ID_Set_Value_D2 )  SetMemVal( STR_EXT2, pExtLoc_DF, pExtVal_D5, ID_ExtVal_D5 );

		// slider / progress bar: they use the same memory space for the control value, so no need to copy the value, just send it back to the client 
		else if (*((unsigned short*) pData) == ID_Slider_FA) 
		{
			FxPCommand( FxPC_CtlUpdate, ID_Progress_FB, 0 );
		}

		// miscellaneous control test dialog 
		else if (*((unsigned short*) pData) == ID_FlexiPanelLogo_F5) ShowMessage_ImagePressedEETest_F6;

	}

	// response from message box
	else if ( EventID==FxPE_MsgResp ) 
	{
		// update button pressed display
		*pcButtonPressed_F8 = *((unsigned char*) pData) ;
		FxPCommand( FxPC_CtlUpdate, ID_ButtonPressed_F8, 0 );
	}

}

void SetDigitalOutputs( void )
{
	AN0Pin = *pAN0_11;
	AN1Pin = *pAN1_12;
	AN2Pin = *pAN2_13;
	AN3Pin = *pAN3_14;
	AN4Pin = *pAN4_15;
	AN5Pin = *pAN5_16;
	AN6Pin = *pAN6_17;
	AN7Pin = *pAN7_18;
	AN8Pin = *pAN8_19;
	AN9Pin = *pAN9_1A;
	AN10Pin = *pAN10_1B;
	AN11Pin = *pAN11_1C;
	CCP1Pin = *pCCP1_1D;
	CCP2Pin = *pCCP2_1E;
	CCP3Pin = *pCCP3_1F;
	CCP4Pin = *pCCP4_20;
	CCP5Pin = *pCCP5_21;
	INT0Pin = *pINT0_22;
	INT1Pin = *pINT1_23;
	SCLPin = *pSCL_26;
	SDAPin = *pSDA_24;
	SDOPin = *pSDO_25;
	TxDPin = *pTxD_27;
	RxDPin = *pRxD_28;
}

void SetParallelOutputs( void )
{
	// Parallel output A
	switch (*pcPAN_84)
	{
		case 0:  // PA7
			DirPA7( DirOutput );
			SetPA7( *pcPAVal_80 );
			break;
		case 1:  // PA6
			DirPA6( DirOutput );
			SetPA6( *pcPAVal_80 );
			break;
		case 2:  // PA5
			DirPA5( DirOutput );
			SetPA5( *pcPAVal_80 );
			break;
		case 3:  // PA4
			DirPA4( DirOutput );
			SetPA4( *pcPAVal_80 );
			break;
		case 4:  // PA3
			DirPA3( DirOutput );
			SetPA3( *pcPAVal_80 );
			break;
		case 5:  // PA2
			DirPA2( DirOutput );
			SetPA2( *pcPAVal_80 );
			break;
	}

	// Parallel output B
	switch (*pcPBN_83)
	{
		case 0:  // PB5
			DirPB5( DirOutput );
			SetPB5( *pcPBVal_81 );
			break;
		case 1:  // PB4
			DirPB4( DirOutput );
			SetPB4( *pcPBVal_81 );
			break;
		case 2:  // PB3
			DirPB3( DirOutput );
			SetPB3( *pcPBVal_81 );
			break;
		case 3:  // PB2
			DirPB2( DirOutput );
			SetPB2( *pcPBVal_81 );
			break;
	}


	// Parallel output C
	switch (*pcPCN_85)
	{
		case 0:  // PC5
			DirPC5( DirOutput );
			SetPC5( *pcPCVal_82 );
			break;
		case 1:  // PC4
			DirPC4( DirOutput );
			SetPC4( *pcPCVal_82 );
			break;
		case 2:  // PC3
			DirPC3( DirOutput );
			SetPC3( *pcPCVal_82 );
			break;
		case 3:  // PC2
			DirPC2( DirOutput );
			SetPC2( *pcPCVal_82 );
			break;
	}

}

// set LEDs according to dialog controls
void SetLEDs( void )
{
	LedGreen = *pGreen_LED_AB ? LedGreenOn : LedGreenOff;
	LedRed = *pRed_Led_AC ? LedRedOn : LedRedOff;
}

// Convert hex ASCII characters to hex values
unsigned short ValFromHextText( unsigned char * pHexText )
{
	unsigned short RetVal = 0;
	while (*pHexText)
	{
		RetVal = RetVal << 4;
		RetVal = RetVal + (*pHexText) - '0';
		if (*pHexText > 'Z') RetVal -= 0x20;  // a-f adjustment to A-F
		if (*pHexText > '9') RetVal -= 0x07;  // A-F adjustment
		pHexText++;
	}
	return RetVal;
}

// retrieve an item from memory, convert to hex and display
void GetMemVal( unsigned char Storage, unsigned char * pLocText, unsigned char * pValText, unsigned short CtlID )
{
	overlay unsigned char cVal;
	overlay unsigned short Addr = ValFromHextText( pLocText );
	GetBytes( Storage, Addr, &cVal, 1  );
	pValText[2] = 0;
	pValText[1] = (cVal & 0x0F) + '0';
	if (pValText[1] > '9') pValText[1] += 0x07;   // A-F adjustment
	cVal = cVal >> 4;
	pValText[0] = (cVal & 0x0F) + '0';
	if (pValText[0] > '9') pValText[0] += 0x07;   // A-F adjustment
	FxPCommand( FxPC_CtlUpdate, CtlID, 0 );
}

// get new value from UI and set in memory; set text to OK in confirmation
void SetMemVal( unsigned char Storage, unsigned char * pLocText, unsigned char * pValText, unsigned short CtlID )
{
	overlay unsigned char cVal = ValFromHextText( pValText );
	overlay unsigned short Addr = ValFromHextText( pLocText );
	SetBytes( Storage, Addr, 0, &cVal, 1  );
	pValText[0] = 'O';   
	pValText[1] = 'K';   
	pValText[2] = 0;
	FxPCommand( FxPC_CtlUpdate, CtlID, 0 );
}

